@@ -1,6 +1,7 @@
 import json
 import logging
 from pathlib import Path
+import glob
 
 import pandas as pd
 from ..exceptions import SABaseException
@@ -8,28 +9,34 @@ logger = logging.getLogger("superannotate-python-sdk")
 
 
 def df_to_annotations(df, output_dir):
-    """Converts and saves pandas DataFrame annotation info (see aggregate_annotations_as_df) in output_dir
-    The DataFrame should have columns: "imageName", "classNmae", "attributeGroupName", "attributeName", "type", "error", "locked", "visible", trackingId", "probability", "pointLabels", "meta", "commentResolved", "classColor"
+    """Converts and saves pandas DataFrame annotation info (see aggregate_annotations_as_df)
+    in output_dir.
+    The DataFrame should have columns: "imageName", "className", "attributeGroupName",
+    "attributeName", "type", "error", "locked", "visible", trackingId", "probability",
+    "pointLabels", "meta", "commentResolved", "classColor", "groupId"
 
     Currently only works for Vector projects.
 
-    :param df: pandas DataFrame
+    :param df: pandas DataFrame of annotations possibly created by aggregate_annotations_as_df
     :type df: pandas.DataFrame
-    :param include_classes_wo_annotations: enables inclusion of classes without annotations info
-    :type include_classes_wo_annotations: bool
+    :param output_dir: output dir for annotations and classes.json
+    :type output_dir: str or Pathlike
 
-    :return: DataFrame on annotations with columns: ["imageName", "classNmae", "attributeGroupName", "attributeName", "type", "error", "locked", "visible", trackingId", "probability", "pointLabels", "meta", "commentResolved"]
-    :rtype: pandas DataFrame
     """
 
     project_suffix = "objects.json"
-    images = df["imageName"].unique()
+    images = df["imageName"].dropna().unique()
     for image in images:
+        image_status = None
+        image_pinned = None
+        image_height = None
+        image_width = None
         image_df = df[df["imageName"] == image]
         image_annotation = []
-        instances = image_df["instanceId"].unique()
+        instances = image_df["instanceId"].dropna().unique()
         for instance in instances:
             instance_df = image_df[image_df["instanceId"] == instance]
+            # print(instance_df["instanceId"])
             annotation_type = instance_df.iloc[0]["type"]
             annotation_meta = instance_df.iloc[0]["meta"]
 
@@ -50,6 +57,7 @@ def df_to_annotations(df, output_dir):
             )
             instance_annotation["trackingId"] = instance_df.iloc[0]["trackingId"
                                                                    ]
+            instance_annotation["groupId"] = int(instance_df.iloc[0]["groupId"])
             instance_annotation.update(annotation_meta)
             for _, row in instance_df.iterrows():
                 if row["attributeGroupName"] is not None:
@@ -60,6 +68,26 @@ def df_to_annotations(df, output_dir):
                         }
                     )
             image_annotation.append(instance_annotation)
+            image_width = image_width or instance_df.iloc[0]["imageWidth"]
+            image_height = image_height or instance_df.iloc[0]["imageHeight"]
+            image_pinned = image_pinned or instance_df.iloc[0]["imagePinned"]
+            image_status = image_status or instance_df.iloc[0]["imageStatus"]
+
+        comments = image_df[image_df["type"] == "comment"]
+        for _, comment in comments.iterrows():
+            comment_json = {"type": "comment"}
+            comment_json.update(comment["meta"])
+            comment_json["resolved"] = comment["commentResolved"]
+            image_annotation.append(comment_json)
+
+        meta = {
+            "type": "meta",
+            "width": int(image_width),
+            "height": int(image_height),
+            "status": image_status,
+            "pinned": bool(image_pinned)
+        }
+        image_annotation.append(meta)
         json.dump(
             image_annotation,
             open(output_dir / f"{image}___{project_suffix}", "w"),
@@ -113,18 +141,28 @@ def aggregate_annotations_as_df(
     project_root,
     include_classes_wo_annotations=False,
     include_comments=False,
+    include_tags=False,
     verbose=True
 ):
     """Aggregate annotations as pandas dataframe from project root.
 
     :param project_root: export path of the project
     :type project_root: Pathlike (str or Path)
-    :param include_classes_wo_annotations: enables inclusion of classes info that have no instances in annotations
+    :param include_classes_wo_annotations: enables inclusion of classes info
+                                           that have no instances in annotations
     :type include_classes_wo_annotations: bool
     :param include_comments: enables inclusion of comments info as commentResolved column
     :type include_comments: bool
+    :param include_tags: enables inclusion of tags info as tag column
+    :type include_tags: bool
 
-    :return: DataFrame on annotations with columns: "imageName", "instanceId" className", "attributeGroupName", "attributeName", "type", "error", "locked", "visible", "trackingId", "probability", "pointLabels", "meta" (geometry information as string), "commentResolved", "classColor"
+    :return: DataFrame on annotations with columns: "imageName", "instanceId",
+                                                    "className", "attributeGroupName", "attributeName", "type", "error", "locked",
+                                                    "visible", "trackingId", "probability", "pointLabels",
+                                                    "meta" (geometry information as string), "commentResolved", "classColor",
+                                                    "groupId", "imageWidth", "imageHeight", "imageStatus", "imagePinned",
+                                                    "createdAt", "creatorRole", "creationType", "creatorEmail", "updatedAt",
+                                                    "updatorRole", "updatorEmail", "tag"
     :rtype: pandas DataFrame
     """
 
@@ -135,6 +173,10 @@ def aggregate_annotations_as_df(
 
     annotation_data = {
         "imageName": [],
+        "imageHeight": [],
+        "imageWidth": [],
+        "imageStatus": [],
+        "imagePinned": [],
         "instanceId": [],
         "className": [],
         "attributeGroupName": [],
@@ -148,10 +190,20 @@ def aggregate_annotations_as_df(
         "pointLabels": [],
         "meta": [],
         "classColor": [],
+        "groupId": [],
+        "createdAt": [],
+        "creatorRole": [],
+        "creationType": [],
+        "creatorEmail": [],
+        "updatedAt": [],
+        "updatorRole": [],
+        "updatorEmail": []
     }
 
     if include_comments:
         annotation_data["commentResolved"] = []
+    if include_tags:
+        annotation_data["tag"] = []
 
     classes_path = Path(project_root) / "classes" / "classes.json"
     if not classes_path.is_file():
@@ -160,6 +212,18 @@ def aggregate_annotations_as_df(
             " not found. Please provide correct project export root"
         )
     classes_json = json.load(open(classes_path))
+    class_name_to_color = {}
+    class_group_name_to_values = {}
+    for annotation_class in classes_json:
+        name = annotation_class["name"]
+        color = annotation_class["color"]
+        class_name_to_color[name] = color
+        class_group_name_to_values[name] = {}
+        for attribute_group in annotation_class["attribute_groups"]:
+            class_group_name_to_values[name][attribute_group["name"]] = []
+            for attribute in attribute_group["attributes"]:
+                class_group_name_to_values[name][attribute_group["name"]
+                                                ].append(attribute["name"])
 
     def __append_annotation(annotation_dict):
         for annotation_key in annotation_data:
@@ -170,43 +234,65 @@ def aggregate_annotations_as_df(
             else:
                 annotation_data[annotation_key].append(None)
 
+    def __get_image_metadata(image_name, annotations):
+        image_metadata = {"imageName": image_name}
+        for annotation in annotations:
+            if "type" in annotation and annotation["type"] == "meta":
+                image_metadata["imageHeight"] = annotation.get("height")
+                image_metadata["imageWidth"] = annotation.get("width")
+                image_metadata["imageStatus"] = annotation.get("status")
+                image_metadata["imagePinned"] = annotation.get("pinned")
+                break
+        return image_metadata
+
+    def __get_user_metadata(annotation):
+        annotation_created_at = pd.to_datetime(annotation.get("createdAt"))
+        annotation_created_by = annotation.get("createdBy")
+        annotation_creator_email = None
+        annotation_creator_role = None
+        if annotation_created_by:
+            annotation_creator_email = annotation_created_by.get("email")
+            annotation_creator_role = annotation_created_by.get("role")
+        annotation_creation_type = annotation.get("creationType")
+        annotation_updated_at = pd.to_datetime(annotation.get("updatedAt"))
+        annotation_updated_by = annotation.get("updatedBy")
+        annotation_updator_email = None
+        annotation_updator_role = None
+        if annotation_updated_by:
+            annotation_updator_email = annotation_updated_by.get("email")
+            annotation_updator_role = annotation_updated_by.get("role")
+        user_metadata = {
+            "createdAt": annotation_created_at,
+            "creatorRole": annotation_creator_role,
+            "creatorEmail": annotation_creator_email,
+            "creationType": annotation_creation_type,
+            "updatedAt": annotation_updated_at,
+            "updatorRole": annotation_updator_role,
+            "updatorEmail": annotation_updator_email
+        }
+        return user_metadata
+
     annotations_paths = []
 
     for path in Path(project_root).glob('*.json'):
-        if path.name.endswith('___objects.json'
-                             ) or path.name.endswith('___pixel.json'):
-            annotations_paths.append(path)
+        annotations_paths.append(path)
 
     if not annotations_paths:
         logger.warning(
             "No annotations found in project export root %s", project_root
         )
-
+    type_postfix = "___objects.json" if glob.glob(
+        "{}/*___objects.json".format(project_root)
+    ) else "___pixel.json"
     for annotation_path in annotations_paths:
         annotation_json = json.load(open(annotation_path))
-        annotation_image_name = annotation_path.name.split("___")[0]
+        image_name = annotation_path.name.split(type_postfix)[0]
+        image_metadata = __get_image_metadata(image_name, annotation_json)
         annotation_instance_id = 0
         for annotation in annotation_json:
-            annotation_class_name = None
-            annotation_class_color = None
-            attribute_group = None
-            attribute_name = None
-            annotation_type = None
-            annotation_locked = None
-            annotation_visible = None
-            annotation_tracking_id = None
-            annotation_meta = None
-            annotation_error = None
-            annotation_probability = None
-            annotation_point_labels = None
-            comment_meta = None
-            comment_resolved = None
-
             annotation_type = annotation.get("type", "mask")
-
-            if annotation_type in ['meta', 'tag']:
+            if annotation_type == "meta":
                 continue
-
             if annotation_type == "comment":
                 if include_comments:
                     comment_resolved = annotation["resolved"]
@@ -215,35 +301,39 @@ def aggregate_annotations_as_df(
                         "y": annotation["y"],
                         "comments": annotation["comments"]
                     }
-                    __append_annotation(
-                        {
-                            "imageName": annotation_image_name,
-                            "type": annotation_type,
-                            "meta": comment_meta,
-                            "commentResolved": comment_resolved,
-                        }
-                    )
+                    annotation_dict = {
+                        "type": annotation_type,
+                        "meta": comment_meta,
+                        "commentResolved": comment_resolved,
+                    }
+                    user_metadata = __get_user_metadata(annotation)
+                    annotation_dict.update(user_metadata)
+                    annotation_dict.update(image_metadata)
+                    __append_annotation(annotation_dict)
+                continue
+            if annotation_type == "tag":
+                if include_tags:
+                    annotation_tag = annotation["name"]
+                    annotation_dict = {
+                        "type": annotation_type,
+                        "tag": annotation_tag
+                    }
+                    annotation_dict.update(image_metadata)
+                    __append_annotation(annotation_dict)
                 continue
-
-            annotation_instance_id += 1
-
             annotation_class_name = annotation.get("className")
-            if annotation_class_name is not None:
-                for annotation_class in classes_json:
-                    if annotation_class["name"] == annotation_class_name:
-                        annotation_class_color = annotation_class["color"]
-                        break
-                else:
-                    raise SABaseException(
-                        0, "Annotation class not found in classes.json"
-                    )
-
+            if annotation_class_name is None or annotation_class_name not in class_name_to_color:
+                logger.warning(
+                    "Annotation class %s not found in classes json. Skipping.",
+                    annotation_class_name
+                )
+                continue
+            annotation_class_color = class_name_to_color[annotation_class_name]
+            annotation_group_id = annotation.get("groupId")
             annotation_locked = annotation.get("locked")
-
             annotation_visible = annotation.get("visible")
-
             annotation_tracking_id = annotation.get("trackingId")
-
+            annotation_meta = None
             if annotation_type in ["bbox", "polygon", "polyline", "cuboid"]:
                 annotation_meta = {"points": annotation["points"]}
             elif annotation_type == "point":
@@ -263,41 +353,52 @@ def aggregate_annotations_as_df(
                     "connections": annotation["connections"],
                     "points": annotation["points"]
                 }
-
             annotation_error = annotation.get('error')
-
             annotation_probability = annotation.get("probability")
-
             annotation_point_labels = annotation.get("pointLabels")
-
             attributes = annotation.get("attributes")
-
+            user_metadata = __get_user_metadata(annotation)
+            num_added = 0
             if not attributes:
-                __append_annotation(
-                    {
-                        "imageName": annotation_image_name,
-                        "instanceId": annotation_instance_id,
-                        "className": annotation_class_name,
-                        "type": annotation_type,
-                        "locked": annotation_locked,
-                        "visible": annotation_visible,
-                        "trackingId": annotation_tracking_id,
-                        "meta": annotation_meta,
-                        "error": annotation_error,
-                        "probability": annotation_probability,
-                        "pointLabels": annotation_point_labels,
-                        "classColor": annotation_class_color
-                    }
-                )
-
-            for attribute in attributes:
-
-                attribute_group = attribute["groupName"]
-                attribute_name = attribute['name']
-
-                __append_annotation(
-                    {
-                        "imageName": annotation_image_name,
+                annotation_dict = {
+                    "imageName": image_name,
+                    "instanceId": annotation_instance_id,
+                    "className": annotation_class_name,
+                    "type": annotation_type,
+                    "locked": annotation_locked,
+                    "visible": annotation_visible,
+                    "trackingId": annotation_tracking_id,
+                    "meta": annotation_meta,
+                    "error": annotation_error,
+                    "probability": annotation_probability,
+                    "pointLabels": annotation_point_labels,
+                    "classColor": annotation_class_color,
+                    "groupId": annotation_group_id,
+                }
+                annotation_dict.update(user_metadata)
+                annotation_dict.update(image_metadata)
+                __append_annotation(annotation_dict)
+                num_added = 1
+            else:
+                for attribute in attributes:
+                    attribute_group = attribute.get("groupName")
+                    attribute_name = attribute.get('name')
+                    if attribute_group not in class_group_name_to_values[
+                        annotation_class_name]:
+                        logger.warning(
+                            "Annotation class group %s not in classes json. Skipping.",
+                            attribute_group
+                        )
+                        continue
+                    if attribute_name not in class_group_name_to_values[
+                        annotation_class_name][attribute_group]:
+                        logger.warning(
+                            "Annotation class group value %s not in classes json. Skipping.",
+                            attribute_name
+                        )
+                        continue
+                    annotation_dict = {
+                        "imageName": image_name,
                         "instanceId": annotation_instance_id,
                         "className": annotation_class_name,
                         "attributeGroupName": attribute_group,
@@ -310,9 +411,16 @@ def aggregate_annotations_as_df(
                         "error": annotation_error,
                         "probability": annotation_probability,
                         "pointLabels": annotation_point_labels,
-                        "classColor": annotation_class_color
+                        "classColor": annotation_class_color,
+                        "groupId": annotation_group_id,
                     }
-                )
+                    annotation_dict.update(user_metadata)
+                    annotation_dict.update(image_metadata)
+                    __append_annotation(annotation_dict)
+                    num_added += 1
+
+            if num_added > 0:
+                annotation_instance_id += 1
 
     df = pd.DataFrame(annotation_data)
 
@@ -320,11 +428,15 @@ def aggregate_annotations_as_df(
     if include_classes_wo_annotations:
         for class_meta in classes_json:
             annotation_class_name = class_meta["name"]
+            annotation_class_color = class_meta["color"]
 
             if not annotation_class_name in df["className"].unique():
-                __append_annotation({
-                    "className": annotation_class_name,
-                })
+                __append_annotation(
+                    {
+                        "className": annotation_class_name,
+                        "classColor": annotation_class_color
+                    }
+                )
                 continue
 
             class_df = df[df["className"] == annotation_class_name][[
@@ -349,6 +461,7 @@ def aggregate_annotations_as_df(
                         __append_annotation(
                             {
                                 "className": annotation_class_name,
+                                "classColor": annotation_class_color,
                                 "attributeGroupName": attribute_group_name,
                                 "attributeName": attribute_name,
                             }
