@@ -6,27 +6,24 @@ from pathlib import Path
 import boto3
 import cv2
 import numpy as np
-from PIL import Image, ImageDraw
 import requests
+from PIL import Image, ImageDraw
 
+from .. import common
 from ..annotation_helpers import (
-    add_annotation_bbox_to_json, add_annotation_cuboid_to_json,
-    add_annotation_ellipse_to_json, add_annotation_point_to_json,
-    add_annotation_polygon_to_json, add_annotation_polyline_to_json,
-    add_annotation_template_to_json
+    add_annotation_bbox_to_json, add_annotation_comment_to_json,
+    add_annotation_cuboid_to_json, add_annotation_ellipse_to_json,
+    add_annotation_point_to_json, add_annotation_polygon_to_json,
+    add_annotation_polyline_to_json, add_annotation_template_to_json
 )
 from ..api import API
-from ..common import (
-    annotation_status_str_to_int, deprecated_alias, project_type_int_to_str,
-    image_path_to_annotation_paths, hex_to_rgb
-)
 from ..exceptions import SABaseException
 from .annotation_classes import (
     fill_class_and_attribute_ids, fill_class_and_attribute_names,
     get_annotation_classes_id_to_name, get_annotation_classes_name_to_id,
     search_annotation_classes
 )
-from .project import get_project_metadata
+from .project_api import get_project_metadata_bare
 
 logger = logging.getLogger("superannotate-python-sdk")
 
@@ -72,11 +69,13 @@ def search_images(
     :rtype: list of dicts or strs
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     team_id, project_id = project["team_id"], project["id"]
     folder_id = _get_project_root_folder_id(project)  # maybe changed in future
     if annotation_status is not None:
-        annotation_status = annotation_status_str_to_int(annotation_status)
+        annotation_status = common.annotation_status_str_to_int(
+            annotation_status
+        )
 
     result_list = []
     params = {
@@ -113,7 +112,18 @@ def search_images(
             raise SABaseException(
                 response.status_code, "Couldn't search images " + response.text
             )
-    return result_list
+
+    if return_metadata:
+
+        def process_result(x):
+            x["annotation_status"] = common.annotation_status_int_to_str(
+                x["annotation_status"]
+            )
+            return x
+
+        return list(map(process_result, result_list))
+    else:
+        return result_list
 
 
 def get_image_metadata(project, image_name):
@@ -131,10 +141,7 @@ def get_image_metadata(project, image_name):
     for image in images:
         if image["name"] == image_name:
             return image
-    raise SABaseException(
-        0, "Image " + image_name + " doesn't exist in the project " +
-        project["name"]
-    )
+    raise SABaseException(0, "Image " + image_name + " doesn't exist.")
 
 
 def set_image_annotation_status(project, image_name, annotation_status):
@@ -154,7 +161,7 @@ def set_image_annotation_status(project, image_name, annotation_status):
     image = get_image_metadata(project, image_name)
     team_id, project_id, image_id = image["team_id"], image["project_id"
                                                            ], image["id"]
-    annotation_status = annotation_status_str_to_int(annotation_status)
+    annotation_status = common.annotation_status_str_to_int(annotation_status)
     json_req = {
         "annotation_status": annotation_status,
     }
@@ -170,6 +177,40 @@ def set_image_annotation_status(project, image_name, annotation_status):
     return response.json()
 
 
+def add_annotation_comment_to_image(
+    project,
+    image_name,
+    comment_text,
+    comment_coords,
+    comment_author,
+    resolved=False
+):
+    """Add a comment to SuperAnnotate format annotation JSON
+
+    :param project: project name or metadata of the project
+    :type project: str or dict
+    :param image_name: image name
+    :type image: str
+    :param comment_text: comment text
+    :type comment_text: str
+    :param comment_coords: [x, y] coords
+    :type comment_coords: list
+    :param comment_author: comment author email
+    :type comment_author: str
+    :param resolved: comment resolve status
+    :type resolved: bool
+    """
+    annotations = get_image_annotations(project, image_name)["annotation_json"]
+    annotations = add_annotation_comment_to_json(
+        annotations,
+        comment_text,
+        comment_coords,
+        comment_author,
+        resolved=resolved
+    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
+
+
 def add_annotation_bbox_to_image(
     project,
     image_name,
@@ -203,9 +244,7 @@ def add_annotation_bbox_to_image(
         annotation_class_attributes,
         error,
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_polygon_to_image(
@@ -239,9 +278,7 @@ def add_annotation_polygon_to_image(
         annotations, polygon, annotation_class_name,
         annotation_class_attributes, error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_polyline_to_image(
@@ -274,9 +311,7 @@ def add_annotation_polyline_to_image(
         annotations, polyline, annotation_class_name,
         annotation_class_attributes, error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_point_to_image(
@@ -309,9 +344,7 @@ def add_annotation_point_to_image(
         annotations, point, annotation_class_name, annotation_class_attributes,
         error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_ellipse_to_image(
@@ -344,9 +377,7 @@ def add_annotation_ellipse_to_image(
         annotations, ellipse, annotation_class_name,
         annotation_class_attributes, error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_template_to_image(
@@ -386,9 +417,7 @@ def add_annotation_template_to_image(
         annotations, template_points, template_connections,
         annotation_class_name, annotation_class_attributes, error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def add_annotation_cuboid_to_image(
@@ -424,9 +453,7 @@ def add_annotation_cuboid_to_image(
         annotations, cuboid, annotation_class_name, annotation_class_attributes,
         error
     )
-    upload_annotations_from_json_to_image(
-        project, image_name, annotations, verbose=False
-    )
+    upload_image_annotations(project, image_name, annotations, verbose=False)
 
 
 def download_image(
@@ -435,6 +462,7 @@ def download_image(
     local_dir_path=".",
     include_annotations=False,
     include_fuse=False,
+    include_overlay=False,
     variant='original'
 ):
     """Downloads the image (and annotation if not None) to local_dir_path
@@ -466,12 +494,13 @@ def download_image(
         )
 
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     img = get_image_bytes(project, image_name, variant=variant)
+    filepath_save = image_name
     if variant == "lores":
-        image_name += "___lores.jpg"
-    filepath = Path(local_dir_path) / image_name
-    with open(filepath, 'wb') as f:
+        filepath_save += "___lores.jpg"
+    filepath_save = Path(local_dir_path) / filepath_save
+    with open(filepath_save, 'wb') as f:
         f.write(img.getbuffer())
     annotations_filepaths = None
     fuse_path = None
@@ -479,13 +508,19 @@ def download_image(
         annotations_filepaths = download_image_annotations(
             project, image_name, local_dir_path
         )
-        if include_fuse:
-            classes = search_annotation_classes(project, return_metadata=True)
-            project_type = project_type_int_to_str(project["type"])
-            fuse_path = create_fuse_image(filepath, classes, project_type)
-    logger.info("Downloaded image %s to %s.", image_name, filepath)
+        if annotations_filepaths is not None and (
+            include_fuse or include_overlay
+        ):
+            classes = search_annotation_classes(project)
+            fuse_path = create_fuse_image(
+                filepath_save,
+                classes,
+                project["type"],
+                output_overlay=include_overlay
+            )
+    logger.info("Downloaded image %s to %s.", image_name, filepath_save)
 
-    return (str(filepath), annotations_filepaths, fuse_path)
+    return (str(filepath_save), annotations_filepaths, fuse_path)
 
 
 def delete_image(project, image_name):
@@ -572,7 +607,7 @@ def get_image_preannotations(project, image_name):
     team_id, project_id, image_id, folder_id = image["team_id"], image[
         "project_id"], image["id"], image['folder_id']
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     project_type = project["type"]
 
     params = {
@@ -589,13 +624,11 @@ def get_image_preannotations(project, image_name):
         raise SABaseException(response.status_code, response.text)
     res = response.json()
 
-    annotation_classes = search_annotation_classes(
-        project, return_metadata=True
-    )
+    annotation_classes = search_annotation_classes(project)
     annotation_classes_dict = get_annotation_classes_id_to_name(
         annotation_classes
     )
-    if project_type == 1:  # vector
+    if project_type == "Vector":
         res = res['preannotation']
         url = res["url"]
         annotation_json_filename = url.rsplit('/', 1)[-1]
@@ -669,7 +702,7 @@ def get_image_annotations(project, image_name, project_type=None):
         "project_id"], image["id"], image['folder_id']
     if project_type is None:
         if not isinstance(project, dict):
-            project = get_project_metadata(project)
+            project = get_project_metadata_bare(project)
         project_type = project["type"]
     params = {
         'team_id': team_id,
@@ -685,13 +718,11 @@ def get_image_annotations(project, image_name, project_type=None):
         raise SABaseException(response.status_code, response.text)
     res = response.json()
 
-    annotation_classes = search_annotation_classes(
-        project, return_metadata=True
-    )
+    annotation_classes = search_annotation_classes(project)
     annotation_classes_dict = get_annotation_classes_id_to_name(
         annotation_classes
     )
-    if project_type == 1:  # vector
+    if project_type == "Vector":
         url = res["objects"]["url"]
         annotation_json_filename = url.rsplit('/', 1)[-1]
         headers = res["objects"]["headers"]
@@ -752,18 +783,18 @@ def download_image_annotations(project, image_name, local_dir_path):
     :rtype: tuple
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
 
     annotation = get_image_annotations(project, image_name)
 
     if annotation["annotation_json_filename"] is None:
         image = get_image_metadata(project, image_name)
-        logger.info("No annotation found for image %s.", image["name"])
+        logger.warning("No annotation found for image %s.", image["name"])
         return None
     return_filepaths = []
     json_path = Path(local_dir_path) / annotation["annotation_json_filename"]
     return_filepaths.append(str(json_path))
-    if project["type"] == 1:
+    if project["type"] == "Vector":
         with open(json_path, "w") as f:
             json.dump(annotation["annotation_json"], f, indent=4)
     else:
@@ -793,14 +824,14 @@ def download_image_preannotations(project, image_name, local_dir_path):
     :rtype: tuple
     """
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     annotation = get_image_preannotations(project, image_name)
     if annotation["preannotation_json_filename"] is None:
         return (None, )
     return_filepaths = []
     json_path = Path(local_dir_path) / annotation["preannotation_json_filename"]
     return_filepaths.append(str(json_path))
-    if project["type"] == 1:
+    if project["type"] == "Vector":
         with open(json_path, "w") as f:
             json.dump(annotation["preannotation_json"], f)
     else:
@@ -814,8 +845,7 @@ def download_image_preannotations(project, image_name, local_dir_path):
     return tuple(return_filepaths)
 
 
-@deprecated_alias(mask_path="mask")
-def upload_annotations_from_json_to_image(
+def upload_image_annotations(
     project, image_name, annotation_json, mask=None, verbose=True
 ):
     """Upload annotations from JSON (also mask for pixel annotations)
@@ -836,7 +866,7 @@ def upload_annotations_from_json_to_image(
             logger.info("Uploading annotations from %s.", annotation_json)
         annotation_json = json.load(open(annotation_json))
     if not isinstance(project, dict):
-        project = get_project_metadata(project)
+        project = get_project_metadata_bare(project)
     image = get_image_metadata(project, image_name)
     team_id, project_id, image_id, folder_id, image_name = image[
         "team_id"], image["project_id"], image["id"], image['folder_id'], image[
@@ -847,9 +877,7 @@ def upload_annotations_from_json_to_image(
             "Uploading annotations for image %s in project %s.", image_name,
             project["name"]
         )
-    annotation_classes = search_annotation_classes(
-        project, return_metadata=True
-    )
+    annotation_classes = search_annotation_classes(project)
     annotation_classes_dict = get_annotation_classes_name_to_id(
         annotation_classes
     )
@@ -864,53 +892,52 @@ def upload_annotations_from_json_to_image(
         path=f'/image/{image_id}/annotation/getAnnotationUploadToken',
         params=params
     )
-    if response.ok:
-        res = response.json()
-        if project_type == 1:  # vector
-            res = res['objects']
-            s3_session = boto3.Session(
-                aws_access_key_id=res['accessKeyId'],
-                aws_secret_access_key=res['secretAccessKey'],
-                aws_session_token=res['sessionToken']
-            )
-            s3_resource = s3_session.resource('s3')
-            bucket = s3_resource.Bucket(res["bucket"])
-            bucket.put_object(
-                Key=res['filePath'], Body=json.dumps(annotation_json)
-            )
-        else:  # pixel
-            if mask is None:
-                raise SABaseException(0, "Pixel annotation should have mask.")
-            if not isinstance(mask, io.BytesIO):
-                with open(mask, "rb") as f:
-                    mask = io.BytesIO(f.read())
-            res_j = res['pixel']
-            s3_session = boto3.Session(
-                aws_access_key_id=res_j['accessKeyId'],
-                aws_secret_access_key=res_j['secretAccessKey'],
-                aws_session_token=res_j['sessionToken']
-            )
-            s3_resource = s3_session.resource('s3')
-            bucket = s3_resource.Bucket(res_j["bucket"])
-            bucket.put_object(
-                Key=res_j['filePath'], Body=json.dumps(annotation_json)
-            )
-            res_m = res['save']
-            s3_session = boto3.Session(
-                aws_access_key_id=res_m['accessKeyId'],
-                aws_secret_access_key=res_m['secretAccessKey'],
-                aws_session_token=res_m['sessionToken']
-            )
-            s3_resource = s3_session.resource('s3')
-            bucket = s3_resource.Bucket(res_m["bucket"])
-            bucket.put_object(Key=res_m['filePath'], Body=mask)
-    else:
+    if not response.ok:
         raise SABaseException(
             response.status_code, "Couldn't upload annotation. " + response.text
         )
+    res = response.json()
+    if project_type == "Vector":
+        res = res['objects']
+        s3_session = boto3.Session(
+            aws_access_key_id=res['accessKeyId'],
+            aws_secret_access_key=res['secretAccessKey'],
+            aws_session_token=res['sessionToken']
+        )
+        s3_resource = s3_session.resource('s3')
+        bucket = s3_resource.Bucket(res["bucket"])
+        bucket.put_object(Key=res['filePath'], Body=json.dumps(annotation_json))
+    else:  # pixel
+        if mask is None:
+            raise SABaseException(0, "Pixel annotation should have mask.")
+        if not isinstance(mask, io.BytesIO):
+            with open(mask, "rb") as f:
+                mask = io.BytesIO(f.read())
+        res_j = res['pixel']
+        s3_session = boto3.Session(
+            aws_access_key_id=res_j['accessKeyId'],
+            aws_secret_access_key=res_j['secretAccessKey'],
+            aws_session_token=res_j['sessionToken']
+        )
+        s3_resource = s3_session.resource('s3')
+        bucket = s3_resource.Bucket(res_j["bucket"])
+        bucket.put_object(
+            Key=res_j['filePath'], Body=json.dumps(annotation_json)
+        )
+        res_m = res['save']
+        s3_session = boto3.Session(
+            aws_access_key_id=res_m['accessKeyId'],
+            aws_secret_access_key=res_m['secretAccessKey'],
+            aws_session_token=res_m['sessionToken']
+        )
+        s3_resource = s3_session.resource('s3')
+        bucket = s3_resource.Bucket(res_m["bucket"])
+        bucket.put_object(Key=res_m['filePath'], Body=mask)
 
 
-def create_fuse_image(image, classes_json, project_type, in_memory=False):
+def create_fuse_image(
+    image, classes_json, project_type, in_memory=False, output_overlay=False
+):
     """Creates fuse for locally located image and annotations
 
     :param image: path to image
@@ -925,7 +952,7 @@ def create_fuse_image(image, classes_json, project_type, in_memory=False):
     :return: path to created fuse image or pillow Image object if in_memory enabled
     :rtype: str of PIL.Image
     """
-    annotation_path = image_path_to_annotation_paths(image, project_type)
+    annotation_path = common.image_path_to_annotation_paths(image, project_type)
     annotation_json = json.load(open(annotation_path[0]))
     if not isinstance(classes_json, list):
         classes_json = json.load(open(classes_json))
@@ -934,16 +961,25 @@ def create_fuse_image(image, classes_json, project_type, in_memory=False):
         if "name" not in ann_class:
             continue
         class_color_dict[ann_class["name"]] = ann_class["color"]
-    image_size = Image.open(image).size
+    pil_image = Image.open(image)
+    image_size = pil_image.size
     fi = np.full((image_size[1], image_size[0], 4), [0, 0, 0, 255], np.uint8)
+    if output_overlay:
+        fi_ovl = np.full(
+            (image_size[1], image_size[0], 4), [0, 0, 0, 255], np.uint8
+        )
+        fi_ovl[:, :, :3] = np.array(pil_image)
     if project_type == "Vector":
         fi_pil = Image.fromarray(fi)
         draw = ImageDraw.Draw(fi_pil)
+        if output_overlay:
+            fi_pil_ovl = Image.fromarray(fi_ovl)
+            draw_ovl = ImageDraw.Draw(fi_pil_ovl)
         for annotation in annotation_json:
             if "className" not in annotation:
                 continue
             color = class_color_dict[annotation["className"]]
-            rgb = hex_to_rgb(color)
+            rgb = common.hex_to_rgb(color)
             fill_color = (rgb[0], rgb[1], rgb[2], 255)
             outline_color = (255, 255, 255, 255)
             if annotation["type"] == "bbox":
@@ -952,22 +988,47 @@ def create_fuse_image(image, classes_json, project_type, in_memory=False):
                     (annotation["points"]["x2"], annotation["points"]["y2"])
                 )
                 draw.rectangle(pt, fill_color, outline_color)
+                if output_overlay:
+                    draw_ovl.rectangle(pt, None, fill_color)
             elif annotation["type"] == "polygon":
                 pts = annotation["points"]
                 draw.polygon(pts, fill_color, outline_color)
+                if output_overlay:
+                    draw_ovl.polygon(pts, None, fill_color)
             elif annotation["type"] == "polyline":
                 pts = annotation["points"]
                 draw.line(pts, fill_color, width=2)
+                if output_overlay:
+                    draw_ovl.line(pts, fill_color, width=2)
             elif annotation["type"] == "point":
                 pts = [
                     annotation["x"] - 2, annotation["y"] - 2,
                     annotation["x"] + 2, annotation["y"] + 2
                 ]
                 draw.ellipse(pts, fill_color, outline_color)
+                if output_overlay:
+                    draw_ovl.ellipse(pts, None, fill_color)
             elif annotation["type"] == "ellipse":
                 temp = np.full(
                     (image_size[1], image_size[0], 3), [0, 0, 0], np.uint8
                 )
+                if output_overlay:
+                    temp_ovl = np.full(
+                        (image_size[1], image_size[0], 3), [0, 0, 0], np.uint8
+                    )
+                    cv2.ellipse(
+                        temp_ovl, (annotation["cx"], annotation["cy"]),
+                        (annotation["rx"], annotation["ry"]),
+                        annotation["angle"], 0, 360, fill_color[:-1], 1
+                    )
+                    new_array_ovl = np.array(fi_pil_ovl)
+                    temp_mask = np.alltrue(temp_ovl != [0, 0, 0], axis=2)
+                    new_array_ovl[:, :, :-1] = np.where(
+                        temp_mask[:, :, np.newaxis], temp_ovl,
+                        new_array_ovl[:, :, :-1]
+                    )
+                    fi_pil_ovl = Image.fromarray(new_array_ovl)
+                    draw_ovl = ImageDraw.Draw(fi_pil_ovl)
                 cv2.ellipse(
                     temp, (annotation["cx"], annotation["cy"]),
                     (annotation["rx"], annotation["ry"]), annotation["angle"],
@@ -994,6 +1055,8 @@ def create_fuse_image(image, classes_json, project_type, in_memory=False):
                 for pt in pts:
                     pt_e = [pt["x"] - 2, pt["y"] - 2, pt["x"] + 2, pt["y"] + 2]
                     draw.ellipse(pt_e, fill_color, fill_color)
+                    if output_overlay:
+                        draw_ovl.ellipse(pt_e, fill_color, fill_color)
                     pt_dict[pt["id"]] = [pt["x"], pt["y"]]
                 connections = annotation["connections"]
                 for connection in connections:
@@ -1002,24 +1065,39 @@ def create_fuse_image(image, classes_json, project_type, in_memory=False):
                         fill_color,
                         width=1
                     )
+                    if output_overlay:
+                        draw_ovl.line(
+                            pt_dict[connection["from"]] +
+                            pt_dict[connection["to"]],
+                            fill_color,
+                            width=1
+                        )
     else:
         annotation_mask = np.array(Image.open(annotation_path[1]))
-        print(annotation_mask.shape, annotation_mask.dtype)
+        # print(annotation_mask.shape, annotation_mask.dtype)
         for annotation in annotation_json:
             if "className" not in annotation or "parts" not in annotation:
                 continue
             color = class_color_dict[annotation["className"]]
-            rgb = hex_to_rgb(color)
+            rgb = common.hex_to_rgb(color)
             fill_color = (rgb[0], rgb[1], rgb[2], 255)
             for part in annotation["parts"]:
                 part_color = part["color"]
-                part_color = list(hex_to_rgb(part_color)) + [255]
+                part_color = list(common.hex_to_rgb(part_color)) + [255]
                 temp_mask = np.alltrue(annotation_mask == part_color, axis=2)
                 fi[temp_mask] = fill_color
         fi_pil = Image.fromarray(fi)
 
     if in_memory:
-        return fi_pil
+        if output_overlay:
+            return (fi_pil, fi_pil_ovl)
+        else:
+            return (fi_pil, )
     fuse_path = str(image) + "___fuse.png"
     fi_pil.save(fuse_path)
-    return fuse_path
+    if output_overlay:
+        overlay_path = str(image) + "___overlay.jpg"
+        fi_pil_ovl.convert("RGB").save(overlay_path, subsampling=0, quality=100)
+        return (fuse_path, overlay_path)
+    else:
+        return (fuse_path, )
