@@ -1,5 +1,13 @@
-class SuperviselyConverter(object):
+import logging
+
+logger = logging.getLogger("superannotate-python-sdk")
+
+
+class SuperviselyConverter():
     def __init__(self, args):
+        logger.warning(
+            "Beta feature. Supervisely to SuperAnnotate annotation format converter is in BETA state."
+        )
 
         self.dataset_name = args.dataset_name
         self.project_type = args.project_type
@@ -7,12 +15,4 @@ class SuperviselyConverter(object):
         self.output_dir = args.output_dir
         self.export_root = args.export_root
         self.direction = args.direction
-
-    def set_output_dir(self, output_dir_):
-        self.output_dir = output_dir_
-
-    def set_export_root(self, export_root_):
-        self.export_root = export_root_
-
-    def set_dataset_name(self, dname):
-        self.dataset_name = dname
\ No newline at end of file
+        self.platform = args.platform
\ No newline at end of file
