@@ -1,4 +1,4 @@
-class VocConverter(object):
+class VocConverter():
     def __init__(self, args):
 
         self.project_type = args.project_type
@@ -7,12 +7,4 @@ class VocConverter(object):
         self.output_dir = args.output_dir
         self.task = args.task
         self.direction = args.direction
-
-    def set_output_dir(self, output_dir):
-        self.output_dir = output_dir
-
-    def set_export_root(self, export_dir):
-        self.export_root = export_dir
-
-    def set_dataset_name(self, dname):
-        self.dataset_name = dname
\ No newline at end of file
+        self.platform = args.platform
