@@ -6,6 +6,8 @@ from .voc_converter import VocConverter
 from .voc_to_sa_pixel import voc_instance_segmentation_to_sa_pixel
 from .voc_to_sa_vector import voc_object_detection_to_sa_vector, voc_instance_segmentation_to_sa_vector
 
+from ....common import dump_output
+
 
 class VocObjectDetectionStrategy(VocConverter):
     name = "ObjectDetection converter"
@@ -24,38 +26,24 @@ class VocObjectDetectionStrategy(VocConverter):
                 elif self.task == "instance_segmentation":
                     self.conversion_algorithm = voc_instance_segmentation_to_sa_vector
             elif self.project_type == "Pixel":
-                if self.task == "object_detection":
-                    raise NotImplementedError("Doesn't support yet")
-                elif self.task == "instance_segmentation":
+                if self.task == "instance_segmentation":
                     self.conversion_algorithm = voc_instance_segmentation_to_sa_pixel
 
     def __str__(self):
         return '{} object'.format(self.name)
 
-    def from_sa_format(self):
-        pass
-
     def to_sa_format(self):
         sa_classes, sa_jsons, sa_masks = self.conversion_algorithm(
             self.export_root
         )
-        with open(
-            os.path.join(self.output_dir, "classes", "classes.json"), "w"
-        ) as fp:
-            json.dump(sa_classes, fp, indent=2)
-
-        for sa_json_name, sa_json_value in sa_jsons.items():
-            with open(os.path.join(self.output_dir, sa_json_name), "w") as fp:
-                json.dump(sa_json_value, fp, indent=2)
+        dump_output(self.output_dir, self.platform, sa_classes, sa_jsons)
 
         if self.project_type == 'Pixel':
-            all_files = os.listdir(self.output_dir)
+            all_files = self.output_dir.glob('*.png')
             for file in all_files:
-                if os.path.splitext(file)[1] == '.png':
-                    os.remove(os.path.join(self.output_dir, file))
+                if '___save.png' not in str(file.name):
+                    (self.output_dir / file.name).unlink()
 
             for sa_mask_name, sa_mask_value in sa_masks.items():
                 sa_mask_value = sa_mask_value[:, :, ::-1]
-                cv2.imwrite(
-                    os.path.join(self.output_dir, sa_mask_name), sa_mask_value
-                )
+                cv2.imwrite(str(self.output_dir / sa_mask_name), sa_mask_value)
